@@ -1,7 +1,5 @@
-# Author: David Goodger
-# Contact: goodger@python.org
-# Revision: $Revision: 1.39 $
-# Date: $Date: 2004/07/29 14:57:28 $
+# $Id: __init__.py 8267 2019-07-20 19:50:51Z grubert $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -13,7 +11,7 @@ Package Structure
 Modules:
 
 - __init__.py: Contains component base classes, exception classes, and
-  Docutils `__version__`.
+  Docutils version information.
 
 - core.py: Contains the ``Publisher`` class and ``publish_*()`` convenience
   functions.
@@ -28,12 +26,6 @@ Modules:
 - statemachine.py: A finite state machine specialized for
   regular-expression-based text filters.
 
-- urischemes.py: Contains a complete mapping of known URI addressing
-  scheme names to descriptions.
-
-- utils.py: Contains the ``Reporter`` system warning class and miscellaneous
-  utilities.
-
 Subpackages:
 
 - languages: Language-specific mappings of terms.
@@ -46,17 +38,54 @@ Subpackages:
 - transforms: Modules used by readers and writers to modify DPS
   doctrees.
 
+- utils: Contains the ``Reporter`` system warning class and miscellaneous
+  utilities used by readers, writers, and transforms.
+
+  utils/urischemes.py: Contains a complete mapping of known URI addressing
+  scheme names to descriptions.
+
+- utils/math: Contains functions for conversion of mathematical notation
+  between different formats (LaTeX, MathML, text, ...).
+
 - writers: Format-specific output translators.
 """
 
+import sys
+from collections import namedtuple
+
+
 __docformat__ = 'reStructuredText'
 
-__version__ = '0.3.5'
-"""``major.minor.micro`` version number.  The micro number is bumped for API
-changes, for new functionality, and for interim project releases.  The minor
-number is bumped whenever there is a significant project release.  The major
-number will be bumped when the project is feature-complete, and perhaps if
-there is a major change in the design."""
+__version__ = '0.15'
+"""Docutils version identifier (complies with PEP 440)::
+
+    major.minor[.micro][releaselevel[serial]][.dev]
+
+For version comparison operations, use `__version_info__` (which see, below)
+rather than parsing the text of `__version__`.
+
+See 'Version Numbering' in docs/dev/policies.txt.
+"""
+
+VersionInfo = namedtuple(
+    'VersionInfo', 'major minor micro releaselevel serial release')
+
+__version_info__ = VersionInfo(
+    major=0,
+    minor=15,
+    micro=0,
+    releaselevel='final', # one of 'alpha', 'beta', 'candidate', 'final'
+    # pre-release serial number (0 for final releases and active development):
+    serial=0,
+    release=True # True for official releases and pre-releases
+    )
+"""Comprehensive version information tuple. See 'Version Numbering' in
+docs/dev/policies.txt."""
+
+__version_details__ = 'release'
+"""Optional extra version details (e.g. 'snapshot 2005-05-29, r3410').
+(For development and release status see `__version_info__`.)
+"""
 
 
 class ApplicationError(StandardError): pass
@@ -79,22 +108,32 @@ class SettingsSpec:
 
     - Option group title (string or `None` which implies no group, just a list
       of single options).
-    
+
     - Description (string or `None`).
-    
+
     - A sequence of option tuples.  Each consists of:
 
       - Help text (string)
-      
+
       - List of option strings (e.g. ``['-Q', '--quux']``).
-      
-      - Dictionary of keyword arguments.  It contains arguments to the
-        OptionParser/OptionGroup ``add_option`` method, possibly with the
-        addition of a 'validator' keyword (see the
-        `docutils.frontend.OptionParser.validators` instance attribute).  Runtime
-        settings names are derived implicitly from long option names
+
+      - Dictionary of keyword arguments sent to the OptionParser/OptionGroup
+        ``add_option`` method.
+
+        Runtime setting names are derived implicitly from long option names
         ('--a-setting' becomes ``settings.a_setting``) or explicitly from the
-        'dest' keyword argument.  See optparse docs for more details.
+        'dest' keyword argument.
+
+        Most settings will also have a 'validator' keyword & function.  The
+        validator function validates setting values (from configuration files
+        and command-line option arguments) and converts them to appropriate
+        types.  For example, the ``docutils.frontend.validate_boolean``
+        function, **required by all boolean settings**, converts true values
+        ('1', 'on', 'yes', and 'true') to 1 and false values ('0', 'off',
+        'no', 'false', and '') to 0.  Validators need only be set once per
+        setting.  See the `docutils.frontend.validate_*` functions.
+
+        See the optparse docs for more details.
 
     - More triples of group title, description, options, as many times as
       needed.  Thus, `settings_spec` tuples can be simply concatenated.
@@ -134,15 +173,26 @@ class TransformSpec:
     TransformSpec subclass objects used by `docutils.transforms.Transformer`.
     """
 
+    def get_transforms(self):
+        """Transforms required by this class.  Override in subclasses."""
+        if self.default_transforms != ():
+            import warnings
+            warnings.warn('default_transforms attribute deprecated.\n'
+                          'Use get_transforms() method instead.',
+                          DeprecationWarning)
+            return list(self.default_transforms)
+        return []
+
+    # Deprecated; for compatibility.
     default_transforms = ()
-    """Transforms required by this class.  Override in subclasses."""
-    
+
     unknown_reference_resolvers = ()
     """List of functions to try to resolve unknown references.  Unknown
     references have a 'refname' attribute which doesn't correspond to any
-    target in the document.  Called when FinalCheckVisitor is unable to find a
-    correct target.  The list should contain functions which will try to
-    resolve unknown references, with the following signature::
+    target in the document.  Called when the transforms in
+    `docutils.tranforms.references` are unable to find a correct target.  The
+    list should contain functions which will try to resolve unknown
+    references, with the following signature::
 
         def reference_resolver(node):
             '''Returns boolean: true if resolved, false if not.'''
@@ -171,7 +221,7 @@ class Component(SettingsSpec, TransformSpec):
 
     supported = ()
     """Names for this component.  Override in subclasses."""
-    
+
     def supports(self, format):
         """
         Is `format` supported by this component?
