@@ -1,6 +1,6 @@
 #! /usr/bin/env python
 
-# $Id: test_tables.py 7316 2012-01-19 11:31:58Z milde $
+# $Id: test_tables.py 8241 2018-11-26 21:50:13Z milde $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -10,8 +10,9 @@ Tests for tables.py directives.
 
 from __init__ import DocutilsTestSupport
 
-import os
+import os, sys
 import csv
+import platform
 from docutils.parsers.rst.directives import tables
 
 
@@ -32,6 +33,19 @@ if isinstance(unichr_exception, OverflowError):
 else:
     unichr_exception_string = str(unichr_exception)
 
+# some error messages changed in Python 3.3:
+# CPython has backported to 2.7.4, PyPy has not
+# platform.python_implementation is new in 2.6
+csv_eod_error_str = 'unexpected end of data'
+if sys.version_info < (2,7,4) or platform.python_implementation() == 'PyPy':
+    csv_eod_error_str = 'newline inside string'
+# pypy adds a line number
+if sys.version_info > (2, 6) and platform.python_implementation() == 'PyPy':
+    csv_eod_error_str = 'line 1: ' + csv_eod_error_str
+csv_unknown_url = "'bogus.csv'"
+if sys.version_info < (3,3,2):
+    csv_unknown_url = "bogus.csv"
+
 def null_bytes():
     import csv
     csv_data = open(utf_16_csv, 'rb').read()
@@ -175,11 +189,196 @@ totest['table'] = [
         <literal_block xml:space="preserve">
             .. table:: empty
 """],
+["""\
+.. table::
+    :width: 100 %
+
+    ============ ==============
+    col 1        col 2
+    ============ ==============
+""",
+"""\
+<document source="test data">
+    <table width="100%">
+        <tgroup cols="2">
+            <colspec colwidth="12">
+            <colspec colwidth="14">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            col 1
+                    <entry>
+                        <paragraph>
+                            col 2
+"""],
+["""\
+.. table::
+    :width: 100px
+
+    ============ ==============
+    col 1        col 2
+    ============ ==============
+""",
+"""\
+<document source="test data">
+    <table width="100px">
+        <tgroup cols="2">
+            <colspec colwidth="12">
+            <colspec colwidth="14">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            col 1
+                    <entry>
+                        <paragraph>
+                            col 2
+"""],
+["""\
+.. table::
+    :width: 321
+
+    ============ ==============
+    col 1        col 2
+    ============ ==============
+""",
+"""\
+<document source="test data">
+    <table width="321">
+        <tgroup cols="2">
+            <colspec colwidth="12">
+            <colspec colwidth="14">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            col 1
+                    <entry>
+                        <paragraph>
+                            col 2
+"""],
+["""\
+.. table::
+    :widths: 15, 25
+
+    ============ ==============
+    Columns with custom widths.
+    ============ ==============
+""",
+"""\
+<document source="test data">
+    <table classes="colwidths-given">
+        <tgroup cols="2">
+            <colspec colwidth="15">
+            <colspec colwidth="25">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Columns with
+                    <entry>
+                        <paragraph>
+                            custom widths.
+"""],
+["""\
+.. table::
+    :widths: 10, 20
+
+    +--------------+----------------+
+    | Columns with | custom widths. |
+    +--------------+----------------+
+""",
+"""\
+<document source="test data">
+    <table classes="colwidths-given">
+        <tgroup cols="2">
+            <colspec colwidth="10">
+            <colspec colwidth="20">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Columns with
+                    <entry>
+                        <paragraph>
+                            custom widths.
+"""],
+["""\
+.. table::
+    :widths: auto
+
+    +--------------+-------------------+
+    | Columns with | automatic widths. |
+    +--------------+-------------------+
+""",
+"""\
+<document source="test data">
+    <table classes="colwidths-auto">
+        <tgroup cols="2">
+            <colspec colwidth="14">
+            <colspec colwidth="19">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Columns with
+                    <entry>
+                        <paragraph>
+                            automatic widths.
+"""],
+["""\
+.. table::
+    :widths: grid
+
+    +--------------+-------------------+
+    | Columns with | automatic widths. |
+    +--------------+-------------------+
+""",
+"""\
+<document source="test data">
+    <table classes="colwidths-given">
+        <tgroup cols="2">
+            <colspec colwidth="14">
+            <colspec colwidth="19">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Columns with
+                    <entry>
+                        <paragraph>
+                            automatic widths.
+"""],
+["""\
+.. table::
+    :align: center
+
+    ======  =====
+    Simple  table
+    ======  =====
+""",
+"""\
+<document source="test data">
+    <table align="center">
+        <tgroup cols="2">
+            <colspec colwidth="6">
+            <colspec colwidth="5">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Simple
+                    <entry>
+                        <paragraph>
+                            table
+"""],
 ]
 
 totest['csv-table'] = [
 ["""\
 .. csv-table:: inline with integral header
+   :width: 80%
    :widths: 10, 20, 30
    :header-rows: 1
    :stub-columns: 1
@@ -192,7 +391,7 @@ totest['csv-table'] = [
 """,
 """\
 <document source="test data">
-    <table>
+    <table classes="colwidths-given" width="80%">
         <title>
             inline with integral header
         <tgroup cols="3">
@@ -252,7 +451,7 @@ totest['csv-table'] = [
 """,
 """\
 <document source="test data">
-    <table>
+    <table classes="colwidths-given">
         <title>
             inline with separate header
         <tgroup cols="3">
@@ -440,6 +639,37 @@ u"""\
                             Heiz\xf6lr\xfccksto\xdfabd\xe4mpfung
 """],
 ["""\
+.. csv-table:: center aligned
+   :align: center
+
+   11, 12
+   21, 22
+""",
+"""\
+<document source="test data">
+    <table align="center">
+        <title>
+            center aligned
+        <tgroup cols="2">
+            <colspec colwidth="50">
+            <colspec colwidth="50">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            11
+                    <entry>
+                        <paragraph>
+                            12
+                <row>
+                    <entry>
+                        <paragraph>
+                            21
+                    <entry>
+                        <paragraph>
+                            22
+"""],
+["""\
 .. csv-table:: empty
 """,
 """\
@@ -572,11 +802,11 @@ u"""\
     <system_message level="4" line="1" source="test data" type="SEVERE">
         <paragraph>
             Problems with "csv-table" directive URL "bogus.csv":
-            unknown url type: bogus.csv.
+            unknown url type: %s.
         <literal_block xml:space="preserve">
             .. csv-table:: bad URL
                :url: bogus.csv
-"""],
+""" % csv_unknown_url],
 ["""\
 .. csv-table:: column mismatch
    :widths: 10,20
@@ -627,7 +857,7 @@ u"""\
                :widths: 0 0 0
             \n\
                some, csv, data
-""" % DocutilsTestSupport.exception_data(int, "y")[1][0]],
+""" % DocutilsTestSupport.exception_data(int, u"y")[1][0]],
 ["""\
 .. csv-table:: good delimiter
    :delim: /
@@ -760,12 +990,12 @@ u"""\
     <system_message level="3" line="1" source="test data" type="ERROR">
         <paragraph>
             Error with CSV data in "csv-table" directive:
-            newline inside string
+            %s
         <literal_block xml:space="preserve">
             .. csv-table:: bad CSV data
             \n\
                "bad", \"csv, data
-"""],
+""" % csv_eod_error_str],
 ["""\
 .. csv-table:: bad CSV header data
    :header: "bad", \"csv, data
@@ -777,13 +1007,13 @@ u"""\
     <system_message level="3" line="1" source="test data" type="ERROR">
         <paragraph>
             Error with CSV data in "csv-table" directive:
-            newline inside string
+            %s
         <literal_block xml:space="preserve">
             .. csv-table:: bad CSV header data
                :header: "bad", \"csv, data
             \n\
                good, csv, data
-"""],
+""" % csv_eod_error_str],
 ["""\
 .. csv-table:: bad encoding
    :file: %s
@@ -901,7 +1131,7 @@ totest['list-table'] = [
 """,
 """\
 <document source="test data">
-    <table>
+    <table classes="colwidths-given">
         <title>
             list table with integral header
         <tgroup cols="3">
@@ -953,6 +1183,156 @@ totest['list-table'] = [
                             On a stick!
 """],
 ["""\
+.. list-table:: list table with integral header
+   :widths: auto
+   :header-rows: 1
+   :stub-columns: 1
+
+   * - Treat
+     - Quantity
+     - Description
+   * - Albatross
+     - 2.99
+     - On a stick!
+""",
+"""\
+<document source="test data">
+    <table classes="colwidths-auto">
+        <title>
+            list table with integral header
+        <tgroup cols="3">
+            <colspec colwidth="33" stub="1">
+            <colspec colwidth="33">
+            <colspec colwidth="33">
+            <thead>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Treat
+                    <entry>
+                        <paragraph>
+                            Quantity
+                    <entry>
+                        <paragraph>
+                            Description
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Albatross
+                    <entry>
+                        <paragraph>
+                            2.99
+                    <entry>
+                        <paragraph>
+                            On a stick!
+"""],
+["""\
+.. list-table:: list table with integral header
+   :header-rows: 1
+   :stub-columns: 1
+
+   * - Treat
+     - Quantity
+     - Description
+   * - Albatross
+     - 2.99
+     - On a stick!
+   * - Crunchy Frog
+     - 1.49
+     - If we took the bones out, it wouldn\'t be
+       crunchy, now would it?
+   * - Gannet Ripple
+     - 1.99
+     - On a stick!
+""",
+"""\
+<document source="test data">
+    <table>
+        <title>
+            list table with integral header
+        <tgroup cols="3">
+            <colspec colwidth="33" stub="1">
+            <colspec colwidth="33">
+            <colspec colwidth="33">
+            <thead>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Treat
+                    <entry>
+                        <paragraph>
+                            Quantity
+                    <entry>
+                        <paragraph>
+                            Description
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            Albatross
+                    <entry>
+                        <paragraph>
+                            2.99
+                    <entry>
+                        <paragraph>
+                            On a stick!
+                <row>
+                    <entry>
+                        <paragraph>
+                            Crunchy Frog
+                    <entry>
+                        <paragraph>
+                            1.49
+                    <entry>
+                        <paragraph>
+                            If we took the bones out, it wouldn\'t be
+                            crunchy, now would it?
+                <row>
+                    <entry>
+                        <paragraph>
+                            Gannet Ripple
+                    <entry>
+                        <paragraph>
+                            1.99
+                    <entry>
+                        <paragraph>
+                            On a stick!
+"""],
+["""\
+.. list-table:: center aligned
+   :align: center
+
+   * - 11
+     - 12
+   * - 21
+     - 22
+""",
+"""\
+<document source="test data">
+    <table align="center">
+        <title>
+            center aligned
+        <tgroup cols="2">
+            <colspec colwidth="50">
+            <colspec colwidth="50">
+            <tbody>
+                <row>
+                    <entry>
+                        <paragraph>
+                            11
+                    <entry>
+                        <paragraph>
+                            12
+                <row>
+                    <entry>
+                        <paragraph>
+                            21
+                    <entry>
+                        <paragraph>
+                            22
+"""],
+["""\
 .. list-table::
 
    not a bullet list
@@ -1070,18 +1450,6 @@ totest['list-table'] = [
 ]
 
 
-if csv:
-    # Rewrite csv tests that depend on the output of IOError as it is
-    # platform-dependent before python 2.4 for a unicode path.
-    # Here only needed for python 2.3 on non-windows
-    import sys
-    if sys.version_info < (2, 4) and not sys.platform.startswith('win'):
-        for i in range(len(totest['csv-table'])):
-            if totest['csv-table'][i][1].find("u'bogus.csv'") != -1:
-                totest['csv-table'][i][1] = totest['csv-table'][i][1].replace(
-                        "u'bogus.csv'", "'bogus.csv'")
-
-
 if __name__ == '__main__':
     import unittest
     unittest.main(defaultTest='suite')
