@@ -1,7 +1,7 @@
 #! /usr/bin/env python
 # -*- coding: utf-8 -*-
 
-# $Id: test_inline_markup.py 7668 2013-06-04 12:46:30Z milde $
+# $Id: test_inline_markup.py 8223 2018-07-11 12:12:34Z grubert $
 # Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
@@ -553,6 +553,36 @@ Python ``list``\s use square bracket syntax.
             list
         s use square bracket syntax.
 """],
+[r"""
+Blank after opening `` not allowed.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Blank after opening `` not allowed.
+"""],
+[r"""
+no blank ``after closing``continues`` literal.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        no blank \n\
+        <literal>
+            after closing``continues
+         literal.
+"""],
+[r"""
+dot ``after closing``. is possible.
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        dot \n\
+        <literal>
+            after closing
+        . is possible.
+"""],
 ]
 
 totest['references'] = [
@@ -981,6 +1011,21 @@ long/path /and  /whitespace>`__
         <reference name="embedded URI with whitespace" refuri="http://example.com/long/path/and/whitespace">
             embedded URI with whitespace
 """],
+[r"""
+`embedded URI with escaped whitespace <http://example.com/a\
+long/path\ and/some\ escaped\ whitespace>`__
+
+`<omitted\ reference\ text\ with\ escaped\ whitespace>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="embedded URI with escaped whitespace" refuri="http://example.com/a long/path and/some escaped whitespace">
+            embedded URI with escaped whitespace
+    <paragraph>
+        <reference name="omitted reference text with escaped whitespace" refuri="omitted reference text with escaped whitespace">
+            omitted reference text with escaped whitespace
+"""],
 ["""\
 `embedded email address <jdoe@example.com>`__
 
@@ -1050,7 +1095,7 @@ Relative URIs' reference text can be omitted:
         <reference name="anonymous" refuri="anonymous">
             anonymous
 """],
-["""\
+[r"""
 Escape trailing low-line char in URIs:
 
 `<reference\_>`_
@@ -1069,6 +1114,25 @@ Escape trailing low-line char in URIs:
         <reference name="anonymous_" refuri="anonymous_">
             anonymous_
 """],
+["""\
+Escape other char in URIs:
+
+`<reference\:1>`_
+
+`<anonymous\\call>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Escape other char in URIs:
+    <paragraph>
+        <reference name="reference:1" refuri="reference:1">
+            reference:1
+        <target ids="reference-1" names="reference:1" refuri="reference:1">
+    <paragraph>
+        <reference name="anonymouscall" refuri="anonymouscall">
+            anonymouscall
+"""],
 ]
 
 totest['embedded_aliases'] = [
@@ -1121,19 +1185,46 @@ long  phrase_>`__
         <reference name="embedded alias with whitespace" refname="alias long phrase">
             embedded alias with whitespace
 """],
-[r"""
-`embedded alias with too much whitespace < alias_ >`__
-
-`embedded alias with no preceding whitespace<alias_>`__
+["""\
+`<embedded alias with whitespace_>`__
 """,
 """\
 <document source="test data">
     <paragraph>
-        <reference anonymous="1" name="embedded alias with too much whitespace < alias_ >">
-            embedded alias with too much whitespace < alias_ >
+        <reference name="embedded alias with whitespace" refname="embedded alias with whitespace">
+            embedded alias with whitespace
+"""],
+[r"""
+`no embedded alias (whitespace inside bracket) < alias_ >`__
+
+`no embedded alias (no preceding whitespace)<alias_>`__
+""",
+"""\
+<document source="test data">
     <paragraph>
-        <reference anonymous="1" name="embedded alias with no preceding whitespace<alias_>">
-            embedded alias with no preceding whitespace<alias_>
+        <reference anonymous="1" name="no embedded alias (whitespace inside bracket) < alias_ >">
+            no embedded alias (whitespace inside bracket) < alias_ >
+    <paragraph>
+        <reference anonymous="1" name="no embedded alias (no preceding whitespace)<alias_>">
+            no embedded alias (no preceding whitespace)<alias_>
+"""],
+[r"""
+`anonymous reference <alias\ with\\ escaped \:characters_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="anonymous reference" refname="aliaswith\ escaped :characters">
+            anonymous reference
+"""],
+[r"""
+`anonymous reference <alias\ with\\ escaped \:characters_>`__
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        <reference name="anonymous reference" refname="aliaswith\ escaped :characters">
+            anonymous reference
 """],
 ]
 
@@ -1592,7 +1683,7 @@ lines.
 """],
 [u"""\
 text-*separated*\u2010*by*\u2011*various*\u2012*dashes*\u2013*and*\u2014*hyphens*.
-\u00bf*punctuation*? \u00a1*examples*!\u00a0*\u00a0no-break-space\u00a0*.
+\u00bf*punctuation*? \u00a1*examples*!\u00a0*no-break-space*\u00a0.
 """,
 u"""\
 <document source="test data">
@@ -1624,11 +1715,11 @@ u"""\
             examples
         !\xa0
         <emphasis>
-            \u00a0no-break-space\u00a0
-        .
+            no-break-space
+        \u00a0.
 """],
 # Whitespace characters:
-#  \u180e*MONGOLIAN VOWEL SEPARATOR*\u180e,   fails in Python 2.4
+#  \u180e*MONGOLIAN VOWEL SEPARATOR*\u180e,   fails in Python 2.6
 [u"""\
 text separated by
 *newline*
@@ -1730,6 +1821,110 @@ u"""\
         <emphasis>
             LINE SEPARATOR
 """],
+[u"""\
+inline markup separated by non-ASCII whitespace
+\xa0**NO-BREAK SPACE**\xa0, \xa0``NO-BREAK SPACE``\xa0, \xa0`NO-BREAK SPACE`\xa0,
+\u2000**EN QUAD**\u2000, \u2000``EN QUAD``\u2000, \u2000`EN QUAD`\u2000,
+\u202f**NARROW NBSP**\u202f, \u202f``NARROW NBSP``\u202f, \u202f`NARROW NBSP`\u202f,
+""",                                      
+u"""\
+<document source="test data">
+    <paragraph>
+        inline markup separated by non-ASCII whitespace
+        \xa0
+        <strong>
+            NO-BREAK SPACE
+        \xa0, \xa0
+        <literal>
+            NO-BREAK SPACE
+        \xa0, \xa0
+        <title_reference>
+            NO-BREAK SPACE
+        \xa0,
+        \u2000
+        <strong>
+            EN QUAD
+        \u2000, \u2000
+        <literal>
+            EN QUAD
+        \u2000, \u2000
+        <title_reference>
+            EN QUAD
+        \u2000,
+        \u202f
+        <strong>
+            NARROW NBSP
+        \u202f, \u202f
+        <literal>
+            NARROW NBSP
+        \u202f, \u202f
+        <title_reference>
+            NARROW NBSP
+        \u202f,
+"""],
+[u"""\
+no inline markup due to whitespace inside and behind: *
+newline
+*
+* space * or one of
+*\xa0NO-BREAK SPACE\xa0*
+*\u1680OGHAM SPACE MARK\u1680*
+*\u2000EN QUAD\u2000*
+*\u2001EM QUAD\u2001*
+*\u2002EN SPACE\u2002*
+*\u2003EM SPACE\u2003*
+*\u2004THREE-PER-EM SPACE\u2004*
+*\u2005FOUR-PER-EM SPACE\u2005*
+*\u2006SIX-PER-EM SPACE\u2006*
+*\u2007FIGURE SPACE\u2007*
+*\u2008PUNCTUATION SPACE\u2008*
+*\u2009THIN SPACE\u2009*
+*\u200aHAIR SPACE\u200a*
+*\u202fNARROW NO-BREAK SPACE\u202f*
+*\u205fMEDIUM MATHEMATICAL SPACE\u205f*
+*\u3000IDEOGRAPHIC SPACE\u3000*
+*\u2028LINE SEPARATOR\u2028*
+""",
+u"""\
+<document source="test data">
+    <paragraph>
+        no inline markup due to whitespace inside and behind: *
+        newline
+        *
+        * space * or one of
+        *\xa0NO-BREAK SPACE\xa0*
+        *\u1680OGHAM SPACE MARK\u1680*
+        *\u2000EN QUAD\u2000*
+        *\u2001EM QUAD\u2001*
+        *\u2002EN SPACE\u2002*
+        *\u2003EM SPACE\u2003*
+        *\u2004THREE-PER-EM SPACE\u2004*
+        *\u2005FOUR-PER-EM SPACE\u2005*
+        *\u2006SIX-PER-EM SPACE\u2006*
+        *\u2007FIGURE SPACE\u2007*
+        *\u2008PUNCTUATION SPACE\u2008*
+        *\u2009THIN SPACE\u2009*
+        *\u200aHAIR SPACE\u200a*
+        *\u202fNARROW NO-BREAK SPACE\u202f*
+        *\u205fMEDIUM MATHEMATICAL SPACE\u205f*
+        *\u3000IDEOGRAPHIC SPACE\u3000*
+        *
+        LINE SEPARATOR
+        *"""],
+[u"""\
+no inline markup because of non-ASCII whitespace following /preceding the markup
+**\xa0NO-BREAK SPACE\xa0** ``\xa0NO-BREAK SPACE\xa0`` `\xa0NO-BREAK SPACE\xa0`
+**\u2000EN QUAD\u2000** ``\u2000EN QUAD\u2000`` `\u2000EN QUAD\u2000`
+**\u202fNARROW NBSP\u202f** ``\u202fNARROW NBSP\u202f`` `\u202fNARROW NBSP\u202f`
+""",
+u"""\
+<document source="test data">
+    <paragraph>
+        no inline markup because of non-ASCII whitespace following /preceding the markup
+        **\xa0NO-BREAK SPACE\xa0** ``\xa0NO-BREAK SPACE\xa0`` `\xa0NO-BREAK SPACE\xa0`
+        **\u2000EN QUAD\u2000** ``\u2000EN QUAD\u2000`` `\u2000EN QUAD\u2000`
+        **\u202fNARROW NBSP\u202f** ``\u202fNARROW NBSP\u202f`` `\u202fNARROW NBSP\u202f`\
+"""],
 # « * » ‹ * › « * » ‹ * › « * » ‹ * › French,
 [u"""\
 "Quoted" markup start-string (matched openers & closers) -> no markup:
