@@ -1,9 +1,7 @@
 #! /usr/bin/env python
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3145 $
-# Date: $Date: 2005-03-28 19:52:16 +0200 (Mon, 28 Mar 2005) $
+# $Id: test_substitutions.py 8206 2017-12-18 14:25:07Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -71,11 +69,11 @@ Here's a series of substitution definitions:
 <document source="test data">
     <paragraph>
         Here's a series of substitution definitions:
-    <substitution_definition names="symbol 1">
+    <substitution_definition names="symbol\ 1">
         <image alt="symbol 1" uri="symbol1.png">
-    <substitution_definition names="SYMBOL 2">
+    <substitution_definition names="SYMBOL\ 2">
         <image alt="SYMBOL 2" height="50" uri="symbol2.png" width="100">
-    <substitution_definition names="symbol 3">
+    <substitution_definition names="symbol\ 3">
         <image alt="symbol 3" uri="symbol3.png">
 """],
 ["""\
@@ -84,7 +82,7 @@ Here's a series of substitution definitions:
 """,
 """\
 <document source="test data">
-    <substitution_definition names="very long substitution text, split across lines">
+    <substitution_definition names="very\ long\ substitution\ text,\ split\ across\ lines">
         <image alt="very long substitution text, split across lines" uri="symbol.png">
 """],
 ["""\
@@ -123,11 +121,11 @@ Followed by a paragraph.
 """,
 """\
 <document source="test data">
-    <substitution_definition names="symbol 1">
+    <substitution_definition names="symbol\ 1">
         <image alt="symbol 1" uri="symbol.png">
     <paragraph>
         Followed by a paragraph.
-    <substitution_definition names="symbol 2">
+    <substitution_definition names="symbol\ 2">
         <image alt="symbol 2" uri="symbol.png">
     <comment xml:space="preserve">
     <block_quote>
@@ -223,9 +221,6 @@ No blank line after.
         <paragraph>
             No directive entry for "directive" in module "docutils.parsers.rst.languages.en".
             Trying "directive" as canonical directive name.
-    <system_message level="3" line="8" source="test data" type="ERROR">
-        <paragraph>
-            Directive "directive" not registered (canonical name "directive").
     <system_message level="3" line="8" source="test data" type="ERROR">
         <paragraph>
             Unknown directive type "directive".
@@ -258,6 +253,41 @@ No blank line after.
     <comment xml:space="preserve">
         |
 """],
+["""\
+Elements that are prohibited inside of substitution definitions:
+
+.. |target| replace:: _`target`
+.. |reference| replace:: anonymous__
+.. |auto-numbered footnote| replace:: [#]_
+""",
+"""\
+<document source="test data">
+    <paragraph>
+        Elements that are prohibited inside of substitution definitions:
+    <system_message level="3" line="3" source="test data" type="ERROR">
+        <paragraph>
+            Substitution definition contains illegal element <target>:
+        <literal_block xml:space="preserve">
+            <target ids="target" names="target">
+                target
+        <literal_block xml:space="preserve">
+            .. |target| replace:: _`target`
+    <system_message level="3" line="4" source="test data" type="ERROR">
+        <paragraph>
+            Substitution definition contains illegal element <reference>:
+        <literal_block xml:space="preserve">
+            <reference anonymous="1" name="anonymous">
+                anonymous
+        <literal_block xml:space="preserve">
+            .. |reference| replace:: anonymous__
+    <system_message level="3" line="5" source="test data" type="ERROR">
+        <paragraph>
+            Substitution definition contains illegal element <footnote_reference>:
+        <literal_block xml:space="preserve">
+            <footnote_reference auto="1" ids="id1">
+        <literal_block xml:space="preserve">
+            .. |auto-numbered footnote| replace:: [#]_
+"""],
 ]
 
 
