@@ -1,9 +1,8 @@
 #! /usr/bin/env python
+# -*- coding: utf-8 -*-
 
-# Author: David Goodger
-# Contact: goodger@users.sourceforge.net
-# Revision: $Revision: 3129 $
-# Date: $Date: 2005-03-26 17:21:28 +0100 (Sat, 26 Mar 2005) $
+# $Id: test_docinfo.py 8231 2018-11-20 23:55:14Z milde $
+# Author: David Goodger <goodger@python.org>
 # Copyright: This module has been placed in the public domain.
 
 """
@@ -17,11 +16,19 @@ from docutils.parsers.rst import Parser
 
 def suite():
     parser = Parser()
-    s = DocutilsTestSupport.TransformTestSuite(parser)
+    settings = {'language_code': 'en'}
+    s = DocutilsTestSupport.TransformTestSuite(
+        parser, suite_settings=settings)
     s.generateTests(totest)
+    settings['language_code'] = 'de'
+    s.generateTests(totest_de)
+    settings['language_code'] = 'ru'
+    s.generateTests(totest_ru)
     return s
 
 totest = {}
+totest_de = {}
+totest_ru = {}
 
 totest['bibliographic_field_lists'] = ((DocInfo,), [
 ["""\
@@ -46,7 +53,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
             1
         <date>
             2001-08-11
-        <field>
+        <field classes="parameter-i">
             <field_name>
                 Parameter i
             <field_body>
@@ -88,7 +95,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
                 me@my.org
         <version>
             1
-        <field>
+        <field classes="abstract">
             <field_name>
                 Abstract
             <field_body>
@@ -99,7 +106,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
                         There can only be one "Abstract" field.
         <date>
             2001-08-11
-        <field>
+        <field classes="parameter-i">
             <field_name>
                 Parameter i
             <field_body>
@@ -126,7 +133,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
 """\
 <document source="test data">
     <docinfo>
-        <field>
+        <field classes="author">
             <field_name>
                 Author
             <field_body>
@@ -142,7 +149,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
             <emphasis>
                 simple
              paragraph
-        <field>
+        <field classes="date">
             <field_name>
                 Date
             <field_body>
@@ -153,7 +160,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
                 <system_message level="2" line="3" source="test data" type="WARNING">
                     <paragraph>
                         Cannot extract compound bibliographic field "Date".
-        <field>
+        <field classes="version">
             <field_name>
                 Version
             <field_body>
@@ -216,16 +223,61 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
 """\
 <document source="test data">
     <docinfo>
-        <author>
-            Only One
-        <author>
-            One, Only
+        <authors>
+            <author>
+                Only One
+        <authors>
+            <author>
+                One, Only
 """],
+# [r""":Authors: Me\, Myself; **I**
+# :Authors: Pac\;Man\\; Ms. Pac\Man; Pac\ Man, Jr.
+# :Authors:
+#     Here
+# 
+#     The\re
+# 
+#     *Every\ where*
+# :Authors: - First\\
+#           - Se\ cond
+#           - Thir\d
+# """,
+# """\
+# <document source="test data">
+#     <docinfo>
+#         <authors>
+#             <author>
+#                 Me, Myself
+#             <author>
+#                 I
+#         <authors>
+#             <author>
+#                 Pac;Man\\
+#             <author>
+#                 Ms. PacMan
+#             <author>
+#                 PacMan, Jr.
+#         <authors>
+#             <author>
+#                 Here
+#             <author>
+#                 There
+#             <author>
+#                 <emphasis>
+#                     Everywhere
+#         <authors>
+#             <author>
+#                 First\\
+#             <author>
+#                 Second
+#             <author>
+#                 Third
+# """],
 ["""\
 :Authors:
 
-:Authors: 1. One
-          2. Two
+:Authors: A. Einstein
+          B. Shaw
 
 :Authors:
     -
@@ -244,28 +296,28 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
 """\
 <document source="test data">
     <docinfo>
-        <field>
+        <field classes="authors">
             <field_name>
                 Authors
             <field_body>
                 <system_message level="2" line="1" source="test data" type="WARNING">
                     <paragraph>
                         Cannot extract empty bibliographic field "Authors".
-        <field>
+        <field classes="authors">
             <field_name>
                 Authors
             <field_body>
-                <enumerated_list enumtype="arabic" prefix="" suffix=".">
+                <enumerated_list enumtype="upperalpha" prefix="" suffix=".">
                     <list_item>
                         <paragraph>
-                            One
+                            Einstein
                     <list_item>
                         <paragraph>
-                            Two
+                            Shaw
                 <system_message level="2" line="3" source="test data" type="WARNING">
                     <paragraph>
                         Bibliographic field "Authors" incompatible with extraction: it must contain either a single paragraph (with authors separated by one of ";,"), multiple paragraphs (one per author), or a bullet list with one paragraph (one author) per item.
-        <field>
+        <field classes="authors">
             <field_name>
                 Authors
             <field_body>
@@ -275,7 +327,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
                 <system_message level="2" line="6" source="test data" type="WARNING">
                     <paragraph>
                         Bibliographic field "Authors" incompatible with extraction: it must contain either a single paragraph (with authors separated by one of ";,"), multiple paragraphs (one per author), or a bullet list with one paragraph (one author) per item.
-        <field>
+        <field classes="authors">
             <field_name>
                 Authors
             <field_body>
@@ -288,7 +340,7 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
                 <system_message level="2" line="10" source="test data" type="WARNING">
                     <paragraph>
                         Bibliographic field "Authors" incompatible with extraction: it must contain either a single paragraph (with authors separated by one of ";,"), multiple paragraphs (one per author), or a bullet list with one paragraph (one author) per item.
-        <field>
+        <field classes="authors">
             <field_name>
                 Authors
             <field_body>
@@ -305,34 +357,113 @@ totest['bibliographic_field_lists'] = ((DocInfo,), [
 ["""\
 .. RCS keyword extraction.
 
-:Status: $""" + """RCSfile: test_docinfo.py,v $
+:Status: (some text) $""" + """RCSfile: test_docinfo.py,v $ (more text) 
 :Date: (some text) $""" + """Date: 2002/10/08 01:34:23 $ (more text)
-:Version: $""" + """Revision: 1.1 $ (still more text)
-
-RCS keyword 'RCSfile' doesn't change unless the file name changes,
-so it's safe. The 'Date' keyword changes every time the file is
-checked in to CVS, so the test's expected output text has to be
-derived (hacked) in parallel in order to stay in sync.
+:Date: (some text) $""" + """Date: 2005-03-26T16:21:28.693201Z $ (more text)
+:Version: (some text) $""" + """Revision: 1.1 $ (more text)
 """,
 """\
 <document source="test data">
     <docinfo>
         <status>
-            test_docinfo.py
+            (some text) test_docinfo.py (more text)
         <date>
-            (some text) %s (more text)
+            (some text) 2002-10-08 (more text)
+        <date>
+            (some text) 2005-03-26 (more text)
         <version>
-            1.1 (still more text)
+            (some text) 1.1 (more text)
     <comment xml:space="preserve">
         RCS keyword extraction.
-    <paragraph>
-        RCS keyword 'RCSfile' doesn't change unless the file name changes,
-        so it's safe. The 'Date' keyword changes every time the file is
-        checked in to CVS, so the test's expected output text has to be
-        derived (hacked) in parallel in order to stay in sync.
-""" % (('$' 'Date: 2002/10/08 01:34:23 $')[7:17].replace('/', '-'),)],
+"""],
 ])
 
+totest_de['bibliographic_field_lists'] = ((DocInfo,), [
+[u"""\
+.. Bibliographic element extraction for a German document.
+
+:Zusammenfassung: Abstract 1.
+:Autor: Me
+:Adresse: 123 My Street
+          Example, EX
+:Kontakt: me@my.org
+:Version: 1
+:Datum: 2001-08-11
+:Parameter i: integer
+""",
+u"""\
+<document source="test data">
+    <docinfo>
+        <author>
+            Me
+        <address xml:space="preserve">
+            123 My Street
+            Example, EX
+        <contact>
+            <reference refuri="mailto:me@my.org">
+                me@my.org
+        <version>
+            1
+        <date>
+            2001-08-11
+        <field classes="parameter-i">
+            <field_name>
+                Parameter i
+            <field_body>
+                <paragraph>
+                    integer
+    <topic classes="abstract">
+        <title>
+            Zusammenfassung
+        <paragraph>
+            Abstract 1.
+    <comment xml:space="preserve">
+        Bibliographic element extraction for a German document.
+"""],])
+
+totest_ru['bibliographic_field_lists'] = ((DocInfo,), [
+[u"""\
+.. Bibliographic element extraction for a Russian document.
+
+:аннотация: Abstract 1.
+:автор: Me
+:адрес: 123 My Street
+          Example, EX
+:контакт: me@my.org
+:версия: 1
+:дата: 2001-08-11
+:Parameter i: integer
+""",
+u"""\
+<document source="test data">
+    <docinfo>
+        <author>
+            Me
+        <address xml:space="preserve">
+            123 My Street
+            Example, EX
+        <contact>
+            <reference refuri="mailto:me@my.org">
+                me@my.org
+        <version>
+            1
+        <date>
+            2001-08-11
+        <field classes="parameter-i">
+            <field_name>
+                Parameter i
+            <field_body>
+                <paragraph>
+                    integer
+    <topic classes="abstract">
+        <title>
+            Аннотация
+        <paragraph>
+            Abstract 1.
+    <comment xml:space="preserve">
+        Bibliographic element extraction for a Russian document.
+"""],])
+
 
 if __name__ == '__main__':
     import unittest
